<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

if(! defined('IN_ADMIN'))
{
	exit;
}

define('FORCE_SELECT', 'task_manager');

$task_id = (isset($_GET['t'])) ? intval($_GET['t']) : 0;

function edit_task_form($form_action, $task_name = '', $task_description = '', $task_file = '', $task_minute = -1, $task_hour = -1, $task_day = -1)
{
	global $mode, $root_path, $template;
	
	$template->load('body', 'edit_task_form.html');
	
	//Minutes select box
	for($i = 0; $i < 60; $i++)
	{
		$template->assign_block_vars('minute', array(
			'VALUE'		=>	$i
		));
	}
	
	//Hours select box
	for($i = 0; $i < 24	; $i++)
	{
		$template->assign_block_vars('hour', array(
			'VALUE'		=>	$i
		));
	}
	
	//Days select box
	for($i = 0; $i < 31	; $i++)
	{
		$template->assign_block_vars('day', array(
			'VALUE'		=>	$i
		));
	}
	
	//Task files select box
	if($dh = opendir("$root_path/tasks"))
	{
		while(($file = readdir($dh)) !== false)
		{
			if($file != '.' && $file != '..')
			{
				$template->assign_block_vars('task_file', array(
					'NAME'	=>	$file,
					'VALUE'	=>	str_replace('.php', '', $file)
				));
			}
		}
		
		closedir($dh);
	}
	
	$template->assign_vars(array(
		'TASK_DESCRIPTION'	=>	$task_description,
		'TASK_DAY'			=>	$task_day,
		'TASK_FILE'			=>	$task_file,
		'TASK_HOUR'			=>	$task_hour,
		'TASK_MINUTE'		=>	$task_minute,
		'TASK_NAME'			=>	$task_name,
		
		'L_BUTTON_TEXT'		=>	($mode == 'edit') ? lang('update_task') : lang('create_task'),
		'L_HEADER_TEXT'		=>	($mode == 'edit') ? lang('edit_task') : lang('new_task'),
		
		'FORM_ACTION'		=>	$form_action
	));
	
	$template->display('body');
}

switch($mode)
{
	case 'run_now':
		$sql = "SELECT *
				FROM tasks
				WHERE task_id = $task_id";
		$result = $db->query($sql);
		$task_data = $db->fetch_assoc($result);
		
		$next_run = time();
		//Minutes
		$next_run += ($task_data['minute'] != -1) ? $task_data['minute'] * 60 : 0;
		//Hours
		$next_run += ($task_data['hour'] != -1) ? $task_data['hour'] * 3600 : 0;
		//Days
		$next_run += ($task_data['day'] != -1) ? $task_data['day'] * 86400 : 0;
		
		if(file_exists("{$root_path}/tasks/{$task_data['task_file']}.php"))
		{	
			include("{$root_path}/tasks/{$task_data['task_file']}.php");
			
			$sql = "UPDATE tasks
					SET next_run = $next_run
					WHERE task_id = $task_id";
			$db->query($sql);
		}
		else
		{
			//One for the error log but one also for the general admin log
			//so with any luck an admin will spot it and fix it
			add_log(LOG_ADMIN, 'task_file_no_exist', $task_data['task_name']);
			add_log(LOG_ERROR, 'task_file_no_exist', $task_data['task_name']);
		}
		
		add_log(LOG_ADMIN, 'ran_task', $task_data['task_name']);
		admin_success('index.php?m=task_manager');
	break;
	
	case 'delete':
		define('PAGE_TITLE', 'delete_task');
		
		$task_data = array();
		
		$sql = "SELECT *
				FROM tasks
				WHERE task_id = $task_id";
		$result = $db->query($sql);
		$task_data = $db->fetch_assoc($result);
		if(empty($task_data))
		{
			trigger_error('task_no_exist');
		}
		
		if(isset($_POST['confirm_no']))
		{
			Header('Location: index.php?m=task_manager');
		}
		
		if(isset($_POST['confirm_yes']))
		{
			$sql = "DELETE FROM tasks
					WHERE task_id = $task_id";
			$db->query($sql);
			
			add_log(LOG_ADMIN, 'deleted_task', htmlspecialchars($task_data['task_name']));
			admin_success('index.php?m=task_manager');
		}
		else
		{
			confirm_box("index.php?m=$current_module&amp;mode=delete&amp;t=$task_id", 'confirm_delete_task');
		}
	break;
	
	case 'new':
		define('PAGE_TITLE', 'new_task');
		
		if(isset($_POST['submit']))
		{
			check_required_fields(array('task_name', 'task_file'));
			
			$insert_sql = $db->build_query('INSERT', array(
				'task_name'			=>	htmlspecialchars($_POST['task_name']),
				'task_description'	=>	htmlspecialchars($_POST['task_description']),
				'task_file'			=>	htmlspecialchars($_POST['task_file']),
				'minute'			=>	intval($_POST['minute']),
				'hour'				=>	intval($_POST['hour']),
				'day'				=>	intval($_POST['day']),
			));
			
			$sql = "INSERT INTO tasks $insert_sql";
			$db->query($sql);
			
			add_log(LOG_ADMIN, 'created_task', htmlspecialchars($_POST['task_name']));
			admin_success("index.php?m=task_manager");
		}
		else
		{
			edit_task_form("index.php?m=$current_module&amp;mode=new");
		}
	break;
	
	case 'edit':
		define('PAGE_TITLE', 'edit_task');
		
		if(isset($_POST['submit']))
		{
			check_required_fields(array('task_name', 'task_file'));
			
			$update_sql = $db->build_query('UPDATE', array(
				'task_name'			=>	htmlspecialchars($_POST['task_name']),
				'task_description'	=>	htmlspecialchars($_POST['task_description']),
				'task_file'			=>	htmlspecialchars($_POST['task_file']),
				'minute'			=>	intval($_POST['minute']),
				'hour'				=>	intval($_POST['hour']),
				'day'				=>	intval($_POST['day']),
			));
			
			$sql = "UPDATE tasks
					SET $update_sql
					WHERE task_id = $task_id";
			$db->query($sql);
			
			add_log(LOG_ADMIN, 'updated_task', htmlspecialchars($_POST['task_name']));
			admin_success("index.php?m=task_manager");
		}
		else
		{
			$task_data = array();
			
			$sql = "SELECT *
					FROM tasks
					WHERE task_id = $task_id";
			$result = $db->query($sql);
			$task_data = $db->fetch_assoc($result);
			if(empty($task_data))
			{
				trigger_error('task_no_exist');
			}
			
			edit_task_form("index.php?m=$current_module&amp;mode=edit&amp;t=$task_id", $task_data['task_name'], $task_data['task_description'], $task_data['task_file'], $task_data['minute'], $task_data['hour'], $task_data['day']);
		}
	break;
	
	default:
		trigger_error('no_mode_set');
	break;
}
?>