<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

class auth
{
	var $permissions = array();
	var $forum_permissions = array();
	var $groups = array();
	
	function setup()
	{
		global $cache, $session;
		
		$cached_perms = $cache->get_permission_values();
		foreach($cached_perms['global'] as $k => $v)
		{
			$this->permissions[$k] = PERM_UNSET;
		}
		
		$this->groups = $session->user_data['group_memberships'];
		
		$this->setup_global_perms();
		$this->setup_forum_perms();
	}
	
	function setup_global_perms()
	{
		global $cache, $db, $session;
		
		$group_ids = '';
		$infractions = $raw_perms = $analyse_perms = array();
		
		$num_groups = count($this->groups);
		for($i = 0; $i < $num_groups; $i++)
		{
			$group_ids .= ($i == ($num_groups - 1)) ? $this->groups[$i]['group_id'] : $this->groups[$i]['group_id'].',';
		}
		
		$sql = "SELECT *
				FROM perms p, perm_values v
				WHERE p.perm_name_id = v.perm_name_id
				AND p.group_id IN($group_ids)";
		$result = $db->query($sql);
		while($row = $db->fetch_assoc($result))
		{
			$raw_perms[ $row['perm_name'] ][] = $row;
		}
		
		foreach($raw_perms as $perm_name => $value)
		{
			if(count($value) > 1)
			{
				//We have permission values for more than one group - store in an array to be analysed later
				$analyse_perms[] = $value[0]['perm_name'];
			}
			else
			{
				//Since there is only one value for this permission, we just set it
				//If its UNSET, it is a NO else a YES
				$this->permissions[ $perm_name ] = ($value[0]['perm_value'] == PERM_UNSET) ? PERM_NO : PERM_YES;
			}
		}
		
		for($i = 0; $i < count($analyse_perms); $i++)
		{
			$cur_perm_name = $analyse_perms[$i];
			
			for($j = 0; $j < count($raw_perms[$cur_perm_name]); $j++)
			{
				if($raw_perms[$cur_perm_name][$j]['perm_value'] == PERM_NO)
				{
					$this->permissions[$cur_perm_name] = PERM_NO;
				}
				
				if($raw_perms[$cur_perm_name][$j]['perm_value'] == PERM_YES)
				{
					$this->permissions[$cur_perm_name] = PERM_YES;
				}
			}
		}
		
		$infractions = $cache->get_infractions();
		
		foreach($infractions as $warning_level => $infraction)
		{
			if($session->user_data['user_warnings'] >= $warning_level)
			{
				for($i = 0; $i < count($infraction); $i++)
				{
					$this->permissions[ $infraction[$i]['perm_name'] ] = PERM_NO;
				}
			}
		}
	}
	
	function setup_forum_perms()
	{
		global $cache, $db;
		
		$group_ids = '';
		$analyse_perms = $forum_data = $raw_perms = array();
		
		$sql = "SELECT forum_id
				FROM forums";
		$result = $db->query($sql);
		while($row = $db->fetch_assoc($result))
		{
			$forum_data[] = $row;
			$analyse_perms[ $row['forum_id'] ] = array();
		}
		
		for($i = 0; $i < count($forum_data); $i++)
		{
			$cached_perms = $cache->get_permission_values();
			foreach($cached_perms['forum'] as $k => $v)
			{
				$this->forum_permissions[ $forum_data[$i]['forum_id'] ][$k] = PERM_UNSET;
			}
		}
		
		$num_groups = count($this->groups);
		for($i = 0; $i < $num_groups; $i++)
		{
			$group_ids .= ($i == ($num_groups - 1)) ? $this->groups[$i]['group_id'] : $this->groups[$i]['group_id'].',';
		}
		
		$sql = "SELECT *
				FROM forum_perms p, forum_perm_values v
				WHERE p.perm_name_id = v.perm_name_id
				AND p.group_id IN($group_ids)";
		$result = $db->query($sql);
		while($row = $db->fetch_assoc($result))
		{
			$raw_perms[ $row['forum_id'] ][ $row['perm_name'] ][] = $row;
		}
		
		foreach($raw_perms as $forum_id => $forum_perm_data)
		{
			foreach($forum_perm_data as $perm_name => $perm_data)
			{
				for($i = 0; $i < count($perm_data); $i++)
				{
					if(count($raw_perms[$forum_id][$perm_name]) > 1)
					{
						//We have permission values for more than one group - store in an array to be analysed later
						if(! in_array($raw_perms[$forum_id][$perm_name][0]['perm_name'], $analyse_perms[$forum_id]))
						{
							$analyse_perms[$forum_id][] = $raw_perms[$forum_id][$perm_name][0]['perm_name'];
						}
					}
					else
					{
						//Since there is only one value for this permission, we just set it
						//If its UNSET, it is a NO else a YES
						//$this->permissions[ $perm_name ] = ($raw_perms[$forum_id][$perm_name][0]['perm_value'] == PERM_UNSET) ? PERM_NO : PERM_YES;
						$this->forum_permissions[$forum_id][$perm_name] = $raw_perms[$forum_id][$perm_name][$i]['perm_value'];
					}
				}
			}
		}
		
		foreach($analyse_perms as $forum_id => $perm_name)
		{
			for($i = 0; $i < count($perm_name); $i++)
			{
				foreach($raw_perms[$forum_id] as $perm_name => $perm_data)
				{
					for($j = 0; $j < count($perm_data); $j++)
					{
						if($perm_data[$j]['perm_value'] == PERM_YES)
						{
							$this->forum_permissions[$forum_id][$perm_name] = PERM_YES;
						}
					}
				}
			}
		}
	}
	
	function check($name)
	{
		if(isset($this->permissions[$name]))
		{
			if($this->permissions[$name] == PERM_YES)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		return false;
	}
	
	function forum_check($name, $forum_id)
	{
		if(isset($this->forum_permissions[$forum_id][$name]))
		{
			if($this->forum_permissions[$forum_id][$name] == PERM_YES)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		return false;
	}
}
?>