<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

class cache
{
	var $cache_path;
	var $vars = array();
	var $expires = array();
	
	function cache()
	{
		global $root_path;
		
		$this->cache_path = "{$root_path}/cache/";
	}
	
	function exists($file)
	{
		if(file_exists($this->cache_path.'data_'.$file.'.php'))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function get($file)
	{
		if(file_exists($this->cache_path.'data_'.$file.'.php'))
		{
			include($this->cache_path.'data_'.$file.'.php');
			
			if($this->expires[$file] < time())
			{
				$this->delete($file);
			}
			
			return $this->vars[$file];
		}
	}
	
	function put($file, $data, $custom_expiry = 0)
	{
		//By default, cache items for 14 days (1209600 seconds)
		$expire_time = ($custom_expiry == 0) ? time() + 1209600 : time() + $custom_expiry;
		$string = "<?php\n\$this->vars['$file'] = ". $this->format_array($data) .";\n\$this->expires['$file'] = $expire_time;\n?>";
		
		file_put_contents($this->cache_path.'data_'.$file.'.php', $string);
	}
	
	function delete($file)
	{
		//Do this check so that we don't get errors if we try to delete a
		//cache file that doesn't actually exist
		if(file_exists($this->cache_path.'data_'.$file.'.php'))
		{
			unlink($this->cache_path.'data_'.$file.'.php');
		}
	}
	
	//Function format_array borrowed from phpBB3
	function format_array($array)
	{
		$lines = array();
		foreach ($array as $k => $v)
		{
			if(is_array($v))
			{
				$lines[] = "'$k' => " . $this->format_array($v);
			}
			else if(is_int($v))
			{
				$lines[] = "'$k' => $v";
			}
			else if(is_bool($v))
			{
				$lines[] = "'$k' => " . (($v) ? 'true' : 'false');
			}
			else
			{
				$lines[] = "'$k' => '" . str_replace("'", "\\'", str_replace('\\', '\\\\', $v)) . "'";
			}
		}
		
		return "array(" . implode(',', $lines) . ")";
	}
	
	//==============================================================================================================
	//==============================================================================================================
	//==============================================================================================================
	
	function get_config()
	{
		global $db;
		$config = array();
		
		if(! $this->exists('config'))
		{
			$sql = "SELECT *
					FROM config
					WHERE cache_value = 1";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$config[ $row['config_name'] ] = $row['config_value'];
			}
			
			$this->put('config', $config);
		}
		else
		{
			$config = $this->get('config');
		}
		
		$sql = "SELECT *
				FROM config
				WHERE cache_value = 0";
		$result = $db->query($sql);
		while($row = $db->fetch_assoc($result))
		{
			$config[ $row['config_name'] ] = $row['config_value'];
		}
		
		return $config;
	}
	
	function get_permission_values()
	{
		$perm_values = array();
		
		if(! $this->exists('perm_values'))
		{
			global $db;
			
			$sql = "SELECT *
					FROM perm_values";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$perm_values['global'][ $row['perm_name'] ] = $row;
			}
			
			$sql = "SELECT *
					FROM forum_perm_values";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$perm_values['forum'][ $row['perm_name'] ] = $row;
			}
			
			$this->put('perm_values', $perm_values);
		}
		else
		{
			$perm_values = $this->get('perm_values');
		}
		
		return $perm_values;
	}
	
	function get_ranks()
	{
		$ranks = array('special' => array(), 'normal' => array());
		
		if(! $this->exists('ranks'))
		{
			global $db;
			
			$sql = "SELECT *
					FROM ranks
					ORDER BY rank_posts ASC";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				if($row['rank_special'])
				{
					$ranks['special'][ $row['rank_id'] ] = $row;
				}
				else
				{
					$ranks['normal'][ $row['rank_id'] ] = $row;
				}
			}
			
			$this->put('ranks', $ranks);
		}
		else
		{
			$ranks = $this->get('ranks');
		}
		
		return $ranks;
	}
	
	function get_styles()
	{
		$styles = array();
		
		if(! $this->exists('styles'))
		{
			global $db;
			
			$sql = "SELECT *
					FROM styles";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$styles[ $row['style_id'] ] = $row;
			}
			
			$this->put('styles', $styles);
		}
		else
		{
			$styles = $this->get('styles');
		}
		
		return $styles;
	}
	
	function get_groups()
	{
		$groups = array();
		
		if(! $this->exists('groups'))
		{
			global $db;
			
			$sql = "SELECT *
					FROM groups
					ORDER BY group_name ASC";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$groups[ $row['group_id'] ] = $row;
			}
			
			$this->put('groups', $groups);
		}
		else
		{
			$groups = $this->get('groups');
		}
		
		return $groups;
	}
	
	function get_bad_words()
	{
		global $db;
		
		$words = array();
		
		if(! $this->exists('words'))
		{
			$sql = "SELECT *
					FROM word_censors";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$words[] = $row;
			}
			
			$this->put('words', $words);
		}
		else
		{
			$words = $this->get('words');
		}
		
		return $words;
	}
	
	function get_smilies()
	{
		global $db;
		
		$smilies = array();
		
		if(! $this->exists('smilies'))
		{
			$sql = "SELECT *
					FROM smilies";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$smilies[ $row['smiley_set'] ][] = $row;
			}
			
			$this->put('smilies', $smilies);
		}
		else
		{
			$smilies = $this->get('smilies');
		}
		
		return $smilies;
	}
	
	function get_bans()
	{
		global $db;
		
		$bans = array();
		
		if(! $this->exists('bans'))
		{
			$sql = "SELECT *
					FROM bans";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$bans[] = $row;
			}
			
			$this->put('bans', $bans);
		}
		else
		{
			$bans = $this->get('bans');
		}
		
		return $bans;
	}
	
	function get_infractions()
	{
		global $db;
		
		$infractions = array();
		
		if(! $this->exists('infractions'))
		{
			$sql = "SELECT i.*, v.*
					FROM infractions i, perm_values v
					WHERE i.infraction_affects = v.perm_name_id";
			$result = $db->query($sql);
			while($row = $db->fetch_assoc($result))
			{
				$infractions[ $row['warning_level'] ][] = $row;
			}
			
			$this->put('infractions', $infractions);
		}
		else
		{
			$infractions = $this->get('infractions');
		}
		
		return $infractions;
	}
}
?>