<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

class captcha
{
	var $width;
	var $height;
	var $code;
	var $font;
	var $img;
	
	function captcha($width = 320, $height = 50)
	{
		$this->width = $width;
		$this->height = $height;
		
		$bg_colours = array('FFFFFF', 'FFFFBF', 'F2FBFF', 'FFF9BF', 'FFD9BF', 'FFBFDC', 'EABFFF', 'C8C8FF', 'CCFFBF');
		$rand_colour = array_rand($bg_colours);
		
		$r = hexdec(substr($bg_colours[$rand_colour], 0, 2));
		$g = hexdec(substr($bg_colours[$rand_colour], 2, 2));
		$b = hexdec(substr($bg_colours[$rand_colour], 4, 4));
		
		$this->img = $this->create_blank_image($r, $g, $b);
	}
	
	function create_blank_image($r = 255, $g = 255, $b = 255)
	{
		$img = imagecreatetruecolor($this->width, $this->height);
		
		$white = imagecolorallocate($img, $r, $g, $b);
		imagefill($img, 0, 0, $white);
		
		return $img;
	}
	
	function check($captcha_id, $guess)
	{
		global $db;
		
		$sql = "SELECT *
				FROM captcha
				WHERE captcha_id = '". $db->escape($captcha_id) ."'";
		$result = $db->query($sql);
		$info = $db->fetch_assoc($result);
		
		$this->delete($captcha_id);
		
		if($info['code'] == $guess)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function create_captcha($length = 6)
	{
		global $db;
		
		$captcha_time = time();
		$captcha_id = generate_random_string(32);
		
		$sql = "INSERT INTO captcha (captcha_id, code, captcha_time)
				VALUES('$captcha_id', '". generate_random_string($length) ."', $captcha_time)";
		$db->query($sql);
		
		return $captcha_id;
	}
	
	function show_captcha($captcha_id)
	{
		global $db;
		
		$sql = "SELECT *
				FROM captcha
				WHERE captcha_id = '". $db->escape($captcha_id) ."'";
		$result = $db->query($sql);
		$info = $db->fetch_assoc($result);
		
		$this->font = 'verdana';
		
		header("Content-Type: image/png");
		//if gd
		$this->show_gd_captcha($info['code']);
		//else some other way of making a CAPTCHA
	}
	
	function show_gd_captcha($code)
	{
		global $root_path;
		
		$num_chars = strlen($code);
		
		$colour = imagecolorallocate($this->img, 0, 0, 0);
		$x = rand(5, ($this->width / 10));
		$x_wiggle_room = $this->width / $num_chars;
		
		$this->add_noise();
		
		for($i = 0; $i < $num_chars; $i++)
		{
			$y = rand(27, ($this->height - 10));
			
			imagettftext($this->img, 25, 0, $x, $y, $colour, "{$root_path}/inc/captcha/fonts/$this->font", $code{$i});
			
			$x += rand(15, $x_wiggle_room);
		}
		
		//Debug Info======================================================================================================================
		//imagettftext($this->img, 10, 0, 1, 49, imagecolorallocate($this->img, 0, 0, 0), "{$root_path}/inc/captcha/fonts/$this->font", "Code: $code");
		//================================================================================================================================
		
		imagepng($this->img);
		imagedestroy($this->img);
	}
	
	function add_noise()
	{
		global $config, $root_path;
		
		if(! $config['captcha_noise_bitfield'])
		{
			return;
		}
		
		$enabled_noise = array();
		$available_noise = array(NOISE_LINES, NOISE_SQUARES, NOISE_CIRCLES, NOISE_LETTERS);
		
		//Loop through all the available noises and check they are enabled
		for($i = 0; $i < count($available_noise); $i++)
		{
			if($config['captcha_noise_bitfield'] & $available_noise[$i])
			{
				$enabled_noise[] = $available_noise[$i];
			}
		}
		
		$rand_key = array_rand($enabled_noise);
		$use_noise = $enabled_noise[$rand_key];
		
		switch($use_noise)
		{
			case NOISE_LINES:
				for($i = 0; $i < 20; $i++)
				{
					$colour = imagecolorallocate($this->img, rand(0, 255), rand(0, 255), rand(0, 255));
					
					$x1 = rand(0, $this->width);
					$x2 = rand(0, $this->height);
					$y1 = rand(0, $this->width);
					$y2 = rand(0, $this->height);
					
					imageline($this->img, $x1, $x2, $y1, $y2, $colour);
				}
			break;
			
			case NOISE_SQUARES:
				for($i = 0; $i < 100; $i++)
				{
					$colour = imagecolorallocate($this->img, rand(0, 255), rand(0, 255), rand(0, 255));
					
					$x1 = rand(0, $this->width);
					$x2 = rand(0, $this->height);
					$y1 = $x1 + 5;
					$y2 = $x2 + 5;
					
					imagerectangle($this->img, $x1, $x2, $y1, $y2, $colour);
				}
			break;
			
			case NOISE_CIRCLES:
				for($i = 0; $i < 100; $i++)
				{
					$colour = imagecolorallocate($this->img, rand(0, 255), rand(0, 255), rand(0, 255));
					
					$x = rand(0, $this->width);
					$y = rand(0, $this->height);
					
					imagearc($this->img, $x, $y, 10, 10, 0, 360, $colour);
				}
			break;
			
			case NOISE_LETTERS:
				$colour = imagecolorallocate($this->img, rand(0, 255), rand(0, 255), rand(0, 255));
				
				for($i = 0; $i < 200; $i++)
				{
					$angle = rand(0, 360);
					$colour = imagecolorallocate($this->img, rand(0, 255), rand(0, 255), rand(0, 255));
					
					$x = rand(0, $this->width);
					$y = rand(0, $this->height);
					
					imagettftext($this->img, 10, $angle, $x, $y, $colour, "{$root_path}/inc/captcha/fonts/$this->font", generate_random_string(1));
				}
			break;
		}
	}
	
	function delete($captcha_id)
	{
		global $db;
		
		$sql = "DELETE FROM captcha
				WHERE captcha_id = '". $db->escape($captcha_id) ."'";
		$db->query($sql);
	}
}
?>