<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

//Sends topic review data to the template
//To use in the template, use <!-- INCLUDE topic_review.html -->
function topic_review($topic_id)
{
	global $db, $template;
	
	$sql = "SELECT p.*, u.*
			FROM posts p, users u
			WHERE p.topic_id = $topic_id
			AND p.post_author = u.user_id
			AND p.post_approved = 1
			ORDER BY time_posted DESC
			LIMIT 10";
	$result = $db->query($sql);
	
	while($row = $db->fetch_assoc($result))
	{
		$template->assign_block_vars('topic_review', array(
			'AVATAR_IMG_TEXT'	=>	($row['avatar_type'] != 0) ? generate_avatar_text($row['avatar_type'], $row['avatar_file']) : '',
			'POST_AUTHOR'		=>	$row['username'],
			'POST_TEXT'			=>	newlines(censor_text(parse_smilies(parse_bbcode($row['post_text'])))),
			'POST_TIME'			=>	format_date($row['time_posted'])
		));
	}
}

function create_topic($forum_id, $topic_title, $topic_type, $topic_approved)
{
	global $auth, $db, $root_path, $session;
	
	//Check the user can post as the type specified
	switch($topic_type)
	{
		case TOPIC_STICKY:
			if(! $auth->forum_check('sticky_topic', $forum_id))
			{
				$topic_type = TOPIC_NORMAL;
			}
		break;
		
		case TOPIC_ANNOUNCEMENT:
			if(! $auth->forum_check('announce_topic', $forum_id))
			{
				$topic_type = TOPIC_NORMAL;
			}
		break;
		
		case TOPIC_GLOBAL:
			if(! $auth->forum_check('globalise_topic', $forum_id))
			{
				$topic_type = TOPIC_NORMAL;
			}
		break;
	}
	
	$insert_sql = $db->build_query('INSERT', array(
		'forum_id'			=>	($topic_type != TOPIC_GLOBAL) ? intval($forum_id) : 0,
		'last_post_time'	=>	time(),
		'topic_approved'	=>	(($topic_approved) ? 1 : 0),
		'topic_author'		=>	$session->user_data['user_id'],
		'topic_title'		=>	trim(htmlspecialchars($topic_title)),
		'topic_type'		=>	intval($topic_type)
	));
	
	$sql = "INSERT INTO topics $insert_sql";
	$db->query($sql);
	$topic_id = $db->last_id();
	
	return $topic_id;
}

function create_post($topic_id, $post_subject, $message, $post_approved)
{
	global $auth, $config, $db, $root_path, $session;
	
	$dupe_posts = 0;
	
	$post_text = trim(htmlspecialchars($message));
	$post_md5 = md5($post_text);
	
	//Check for a duplicate post
	$sql = "SELECT *
			FROM posts
			WHERE post_author = ". $session->user_data['user_id'] ."
			AND post_md5 = '$post_md5'
			AND time_posted > ". (time() - $config['duplicate_post_limit']);
	$result = $db->query($sql);
	while($row = $db->fetch_assoc($result))
	{
		$dupe_posts++;
	}
	
	if($dupe_posts > 0)
	{
		define('META_REFRESH', 'index.php');
		trigger_error('duplicate_post');
	}
	
	$insert_sql = $db->build_query('INSERT', array(
		'post_approved'	=>	(($post_approved) ? 1 : 0),
		'post_author'	=>	$session->user_data['user_id'],
		'post_md5'		=>	$post_md5,
		'post_subject'	=>	trim(htmlspecialchars($post_subject)),
		'post_text'		=>	$post_text,
		'time_posted'	=>	time(),
		'topic_id'		=>	intval($topic_id)
	));
	
	$sql = "INSERT INTO posts $insert_sql";
	$db->query($sql);
	$post_id = $db->last_id();
	
	return $post_id;
}
?>