<?php
/*
* --------------------------------------------------------------------------
*                                    NuclearBB
*                              (c) 2007 NuclearBB
* --------------------------------------------------------------------------
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* --------------------------------------------------------------------------
*/

//A small section of code (the IF tag handling) is derived from Smarty and as such, we make no claim to that portion of code
class template
{
	var $data = array();
	var $parsed_code = array();
	var $unparsed_code = array();
	var $cache_path;
	var $style_folder;
	var $style_name;
	var $files = array();
	var $file_path = array();
	var $iterations = array();
	var $force_recompile = false;
	var $stack = array();
	
	function template($style_name)
	{
		global $root_path;
		
		$this->style_name = $style_name;
		$this->cache_path = "$root_path/cache/";
		$this->style_folder = 'styles';
	}
	
	function custom_template($style_folder, $style_name)
	{
		global $root_path;
		
		$this->style_folder = $style_folder;
		$this->style_name = $style_name;
		$this->cache_path = "$root_path/$style_folder/cache/";
	}
	
	function load($handle, $template_file)
	{
		global $root_path;
		
		if(! array_key_exists($handle, $this->files))
		{
			$this->files[$handle] = $template_file;
			$this->file_path[$handle] = $root_path . '/' . $this->style_folder . '/' . $this->style_name . '/' . $template_file;
			
			if(! file_exists($this->file_path[$handle]))
			{
				trigger_error("The handle <em>$handle</em> could not be loaded because the file {$this->file_path[$handle]} does not exist");
			}
		}
		else
		{
			trigger_error("The handle <em>$handle</em> has already been used");
		}
	}
	
	function assign_vars($var_array)
	{
		foreach($var_array as $k => $v)
		{
			$this->data['.'][0][$k] = $v;
		}
	}
	
	function assign_block_vars($block_name, $vars)
	{
		if(array_key_exists($block_name, $this->iterations))
		{
			$this->iterations[$block_name]++;
		}
		else
		{
			$this->iterations[$block_name] = 0;
		}
		
		
		if (strstr($block_name, '.'))
		{
			//Nested Block
			$after_name = substr(strrchr($block_name, '.'), 1);
			$before_name = str_replace(".$after_name", '', $block_name);
			
			for($i = 0; $i < count($this->iterations[$before_name]); $i++)
			{
				$this->data[$before_name][ $this->iterations[$before_name] ][$after_name][$i][] = $vars;
			}
		}
		else
		{
			//Non-nested Block
			$this->data[$block_name][] = $vars;
		}
	}
	
	function parse_directives($handle)
	{
		preg_match_all('#<!-- ([^<].*?) (.*?)?[ ]?-->#', $this->parsed_code[$handle], $directives);
		
		for($i = 0; $i < count($directives[1]); $i++)
		{
			switch($directives[1][$i])
			{
				case 'BEGIN':
					$this->stack[] = $directives[2][$i];
					$this->parse_begin_tag($handle, $directives[2][$i]);
				break;
				
				case 'END':
					array_pop($this->stack);
					$this->parsed_code[$handle] = str_replace($directives[0][$i], '<?php } ?>', $this->parsed_code[$handle]);
				break;
				
				case 'IF':
					$this->parse_if_tag($handle, $directives[2][$i]);
				break;
				
				case 'ELSE':
					$this->parsed_code[$handle] = str_replace($directives[0][$i], '<?php } else { ?>', $this->parsed_code[$handle]);
				break;
				
				case 'ENDIF':
					$this->parsed_code[$handle] = str_replace($directives[0][$i], '<?php } ?>', $this->parsed_code[$handle]);
				break;
				
				case 'INCLUDE':
					$include_handle = str_replace('.html', '_include', $directives[2][$i]);
					
					$include_code = "<?php \$this->load('$include_handle', '{$directives[2][$i]}'); \$this->display('$include_handle'); ?>";
					$this->parsed_code[$handle] = str_replace($directives[0][$i], $include_code, $this->parsed_code[$handle]);
				break;
			}
		}
	}
	
	function parse_begin_tag($handle, $block_name)
	{
		if((count($this->stack)) > 1)
		{
			//Handle nested blocks here
			$before_name = $this->stack[0];
			
			$begin_for = "<?php for(\$i_$block_name = 0; \$i_$block_name < @count(\$this->data['$before_name'][\$i_$before_name]['$block_name'][0]); \$i_$block_name++) { ?>";
			
			$this->parsed_code[$handle] = str_replace("<!-- BEGIN $block_name -->", $begin_for, $this->parsed_code[$handle]);
			
			for($i = 0; $i < @count($this->data[$before_name]); $i++)
			{
				for($j = 0; $j < @count($this->data[$before_name][$i][$block_name][0]); $j++)
				{
					foreach($this->data[$before_name][$i][$block_name][0][$j] as $k => $v)
					{
						$this->parsed_code[$handle] = str_replace('{' . "$before_name.$block_name" . '.' . $k . '}', "<?php echo \$this->data['$before_name'][\$i_$before_name]['$block_name'][0][\$i_$block_name]['$k']; ?>", $this->parsed_code[$handle]);
					}
				}
			}
		}
		else
		{
			//Non-nested blocks
			$begin_for = "<?php for(\$i_$block_name = 0; \$i_$block_name < @count(\$this->data['$block_name']); \$i_$block_name++) { ?>";
			
			$this->parsed_code[$handle] = str_replace("<!-- BEGIN $block_name -->", $begin_for, $this->parsed_code[$handle]);
			
			//Use @count here to surpress any undefiend index errors that may occur
			//due to a programmer not actually assigning any block vars
			for($i = 0; $i < @count($this->data[$block_name]); $i++)
			{
				foreach($this->data[$block_name][$i] as $k => $v)
				{
					$this->parsed_code[$handle] = str_replace('{' . $block_name . '.' . $k . '}', "<?php echo \$this->data['$block_name'][\$i_$block_name]['$k']; ?>", $this->parsed_code[$handle]);
				}
			}
		}
	}
	
	function parse_if_tag($handle, $if_statement)
	{
		global $directive_matches;
		
		// Tokenize args for 'if' tag.
		preg_match_all('/(?:
			"[^"\\\\]*(?:\\\\.[^"\\\\]*)*"         |
			\'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\'     |
			[(),]                                  |
			[^\s(),]+)/x', $if_statement, $match);

		$tokens = $match[0];
		$is_arg_stack = array();

		for($i = 0, $size = sizeof($tokens); $i < $size; $i++)
		{
			$token = &$tokens[$i];

			switch($token)
			{
				case '!==':
				case '===':
				case '<<':
				case '>>':
				case '|':
				case '^':
				case '&':
				case '~':
				case ')':
				case ',':
				case '+':
				case '-':
				case '*':
				case '/':
				case '@':
				break;

				case '==':
				case 'eq':
					$token = '==';
				break;

				case '!=':
				case '<>':
				case 'ne':
				case 'neq':
					$token = '!=';
				break;

				case '<':
				case 'lt':
					$token = '<';
				break;

				case '<=':
				case 'le':
				case 'lte':
					$token = '<=';
				break;

				case '>':
				case 'gt':
					$token = '>';
				break;

				case '>=':
				case 'ge':
				case 'gte':
					$token = '>=';
				break;

				case '&&':
				case 'and':
					$token = '&&';
				break;

				case '||':
				case 'or':
					$token = '||';
				break;

				case '!':
				case 'not':
					$token = '!';
				break;

				case '%':
				case 'mod':
					$token = '%';
				break;

				case '(':
					array_push($is_arg_stack, $i);
				break;
				
				case 'is':
					$is_arg_start = ($tokens[$i-1] == ')') ? array_pop($is_arg_stack) : $i-1;
					$is_arg	= implode('	', array_slice($tokens,	$is_arg_start, $i -	$is_arg_start));

					$new_tokens	= $this->parse_is_expr($is_arg, array_slice($tokens, $i+1));

					array_splice($tokens, $is_arg_start, sizeof($tokens), $new_tokens);

					$i = $is_arg_start;

				// no break
				
				default:
					$parts = explode('.', $token);
					$num_parts = count($parts);
					
					if($num_parts == 2)
					{
						$after_name = substr(strrchr($token, '.'), 1);
						$before_name = str_replace(".$after_name", '', $token);
						
						if((! ($token{0} == "'")) && (! is_numeric($token)))
						{
							$token = str_replace($token, "\$this->data['$before_name'][\$i_$before_name]['$after_name']", $token);
						}
					}
					else if($num_parts == 3)
					{
						$exploded_token = explode('.', $token);
						
						$before_name = $exploded_token[0];
						$after_name = $exploded_token[1];
						$var_name = $exploded_token[2];
						
						if((! ($token{0} == "'")) && (! is_numeric($token)))
						{
							$token = str_replace($token, "\$this->data['$before_name'][\$i_$before_name]['$after_name'][0][\$i_$after_name]['$var_name']", $token);
						}
					}
					else
					{
						if((! ($token{0} == "'")) && (! is_numeric($token)))
						{
							$token = str_replace($token, "\$this->data['.'][0]['$token']", $token);
						}
					}
				break;
			}
		}
		
		$this->parsed_code[$handle] = str_replace("<!-- IF $if_statement -->", '<?php if((' . implode(' ', $tokens) . ')) { ?>', $this->parsed_code[$handle]);
	}
	
	function parse_is_expr($is_arg, $tokens)
	{
		$expr_end =	0;
		$negate_expr = false;

		if (($first_token = array_shift($tokens)) == 'not')
		{
			$negate_expr = true;
			$expr_type = array_shift($tokens);
		}
		else
		{
			$expr_type = $first_token;
		}

		switch ($expr_type)
		{
			case 'even':
				if (@$tokens[$expr_end] == 'by')
				{
					$expr_end++;
					$expr_arg =	$tokens[$expr_end++];
					$expr =	"!(($is_arg	/ $expr_arg) % $expr_arg)";
				}
				else
				{
					$expr =	"!($is_arg % 2)";
				}
			break;

			case 'odd':
				if (@$tokens[$expr_end] == 'by')
				{
					$expr_end++;
					$expr_arg =	$tokens[$expr_end++];
					$expr =	"(($is_arg / $expr_arg)	% $expr_arg)";
				}
				else
				{
					$expr =	"($is_arg %	2)";
				}
			break;

			case 'div':
				if (@$tokens[$expr_end] == 'by')
				{
					$expr_end++;
					$expr_arg =	$tokens[$expr_end++];
					$expr =	"!($is_arg % $expr_arg)";
				}
			break;
		}

		if ($negate_expr)
		{
			$expr =	"!($expr)";
		}

		array_splice($tokens, 0, $expr_end,	$expr);

		return $tokens;
	}
	
	function parse($handle)
	{
		$this->unparsed_code[$handle] = $this->parsed_code[$handle] = @file_get_contents($this->file_path[$handle]);
		
		//Parse directives
		$this->parse_directives($handle);
		
		//Language Strings
		$this->parsed_code[$handle] = preg_replace('#\{L_([a-z0-9\-_]*?)\}#is', "<?php echo (isset(\$this->data['.'][0]['L_\\1'])) ? \$this->data['.'][0]['L_\\1'] : lang('\\1'); ?>", $this->parsed_code[$handle]);
		
		//Placeholders
		$this->parsed_code[$handle] = preg_replace('#\{([a-z0-9\-_]*)\}#is', "<?php echo (isset(\$this->data['.'][0]['\\1'])) ? \$this->data['.'][0]['\\1'] : ''; ?>", $this->parsed_code[$handle]);
		
		$this->parsed_code[$handle] = str_replace('?><?php', '', $this->parsed_code[$handle]);
	}
	
	function display($handle)
	{
		if(! array_key_exists($handle, $this->files))
		{
			trigger_error("Cannot display handle <em>$handle</em> - the template has not been loaded");
		}
		
		if((file_exists($this->cache_path . $this->style_name . '_' . $this->files[$handle] . '.php')) && (! $this->force_recompile))
		{
			include($this->cache_path . $this->style_name . '_' . $this->files[$handle] . '.php');
		}
		else
		{
			$this->parse($handle);
			eval('?>' . $this->parsed_code[$handle]);
			file_put_contents($this->cache_path . $this->style_name . '_' . $this->files[$handle] . '.php', $this->parsed_code[$handle]);
		}
	}
}
?>